"use strict";
(self["webpackChunkxircuits"] = self["webpackChunkxircuits"] || []).push([["node_modules_projectstorm_react-diagrams_dist_index_js-_718a0"],{

/***/ "./node_modules/@projectstorm/react-diagrams/dist/index.js":
/*!*****************************************************************!*\
  !*** ./node_modules/@projectstorm/react-diagrams/dist/index.js ***!
  \*****************************************************************/
/***/ (function(__unused_webpack_module, exports, __webpack_require__) {


var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", ({ value: true }));
const react_diagrams_core_1 = __webpack_require__(/*! @projectstorm/react-diagrams-core */ "./node_modules/@projectstorm/react-diagrams-core/dist/index.js");
const react_diagrams_defaults_1 = __webpack_require__(/*! @projectstorm/react-diagrams-defaults */ "webpack/sharing/consume/default/@projectstorm/react-diagrams-defaults/@projectstorm/react-diagrams-defaults");
const react_diagrams_routing_1 = __webpack_require__(/*! @projectstorm/react-diagrams-routing */ "webpack/sharing/consume/default/@projectstorm/react-diagrams-routing/@projectstorm/react-diagrams-routing");
const react_canvas_core_1 = __webpack_require__(/*! @projectstorm/react-canvas-core */ "webpack/sharing/consume/default/@projectstorm/react-canvas-core/@projectstorm/react-canvas-core?4cd6");
__exportStar(__webpack_require__(/*! @projectstorm/react-diagrams-core */ "./node_modules/@projectstorm/react-diagrams-core/dist/index.js"), exports);
__exportStar(__webpack_require__(/*! @projectstorm/react-diagrams-defaults */ "webpack/sharing/consume/default/@projectstorm/react-diagrams-defaults/@projectstorm/react-diagrams-defaults"), exports);
__exportStar(__webpack_require__(/*! @projectstorm/react-diagrams-routing */ "webpack/sharing/consume/default/@projectstorm/react-diagrams-routing/@projectstorm/react-diagrams-routing"), exports);
/**
 * Construct an engine with the defaults installed
 */
exports["default"] = (options = {}) => {
    const engine = new react_diagrams_core_1.DiagramEngine(options);
    // register model factories
    engine.getLayerFactories().registerFactory(new react_diagrams_core_1.NodeLayerFactory());
    engine.getLayerFactories().registerFactory(new react_diagrams_core_1.LinkLayerFactory());
    engine.getLayerFactories().registerFactory(new react_canvas_core_1.SelectionBoxLayerFactory());
    engine.getLabelFactories().registerFactory(new react_diagrams_defaults_1.DefaultLabelFactory());
    engine.getNodeFactories().registerFactory(new react_diagrams_defaults_1.DefaultNodeFactory()); // i cant figure out why
    engine.getLinkFactories().registerFactory(new react_diagrams_defaults_1.DefaultLinkFactory());
    engine.getLinkFactories().registerFactory(new react_diagrams_routing_1.PathFindingLinkFactory());
    engine.getPortFactories().registerFactory(new react_diagrams_defaults_1.DefaultPortFactory());
    // register the default interaction behaviours
    engine.getStateMachine().pushState(new react_diagrams_core_1.DefaultDiagramState());
    return engine;
};


/***/ })

}]);
//# sourceMappingURL=node_modules_projectstorm_react-diagrams_dist_index_js-_718a0.20f9ad52b47a17ffea59.js.map