/*-----------------------------------------------------------------------------
| Copyright (c) XpressAI GK and Jupyter Development Team.
| Distributed under the terms of the Modified BSD License.
|----------------------------------------------------------------------------*/

/* Refresh Icon */
#main-logo{
  background-image: url(../style/icons/xpress-loading.gif); 
  z-index: 10;
  position: absolute;
  overflow: hidden;
  background-size: 8%;
  background-repeat: no-repeat;
  background-position: center 50%;

}

.orbit{
  visibility: hidden;
}

#main-logo > svg{
  visibility: hidden;
}

.jp-DebuggerWidget {
  color: var(--jp-ui-font-color1);
  background: var(--jp-layout-color1);
  font-size: 14px;
  align-items: center;
  justify-content: left;
  text-align: left;
  overflow-y: auto;
}

.jp-debugger-toolbar-panel {
  flex: 0 0 auto;
}

input[type=number] {
  line-height: 35px;
}

input[type=number]::-webkit-inner-spin-button {
  width: 35px;
  height: 35px;
}

/* Remove outer resizing of run dialog */
.jp-Dialog-content {
  resize: none !important;
}

/* Fix arrow position of HTMLSelect tag on run dialog */
.jp-Dialog-content .f1ozlkqi {
  position: relative;
}